
/***************************************************/
/*                                                 */
/* Styles related to the custom file import widget */
/*                                                 */
/***************************************************/

/* Make file selection button look like other buttons in Django admin. */
.file-google-button {
    font-family: "Roboto", "Lucida Grande", Verdana, Arial, sans-serif;
    font-weight: normal;
    border-radius: 4px;
    border: 1px solid var(--border-color);
    font-size: 13px;
    padding: 4px 15px;
    margin: 4px 0;
    vertical-align: middle;
    background: var(--body-bg);
    box-shadow: 0 -15px 20px -10px rgba(0, 0, 0, 0.15) inset;
    cursor: pointer;
    color: var(--body-fg);
}


/* Make file selection button behave like other buttons in Django admin (if not disabled). */
.file-google-button:focus:enabled,
.file-google-button:hover:enabled {
    border-color: var(--body-quiet-color);
}


/* Disabled button styles */
.file-google-button:disabled {
    color: #888;
}


/* Define the style of the field to display the name of the selected file. */
.file-google-span {
    font-size: 13px;
    margin: 0 0.2em;
    vertical-align: middle;
}
