
<button class="file-google-button" type="button" id="id_import_file" disabled>Select a file ...</button>
<span class="file-google-span" id="id_import_file_span">{{ widget.no_file_txt }}</span>

<input type="file" style="display: none" name="import_file" id="id_import_file_os_file" class="os-file">

<input type="hidden" name="google_file_id" class="google-file" id="id_import_file_google_id">
<input type="hidden" name="google_file_name" class="google-file" id="id_import_file_google_name">
<input type="hidden" name="oauth_token" class="google-file" id="id_import_file_google_oauth_token">
<input type="hidden" name="is_google" class="google-file" id="id_import_file_is_google">

<script>
    const noFileTxt = '{{ widget.no_file_txt }}';
</script>
